/***************************************************************************
                          main.c  -  description
                             -------------------
    begin                : Sun Aug 17 10:52:47 CST 2003
    copyright            : (C) 2003 by Tyler Montbriand
    email                : tsm@accesscomm.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "skinfu.h"

int main(int argc, char *argv[])
{
  int n;
  char ImgDir[300]="./";
  char OutDir[300]="../";
  SDL_Surface *SrcImages[I_END];

  /* Initialize all image pointers to NULL */
  for(n=0; n<I_END; n++)
    SrcImages[n]=NULL;

  if(argc<3)
  {
    fprintf(stderr,"Warning - no input or output directory specified.\n");
    fprintf(stderr,"Use syntax 'skinfu inputdir outputdir'\n\n");
    return(1);
  }

  {
    int len=strlen(argv[1]);    
    strcpy(ImgDir,argv[1]);    
    if(ImgDir[len-1]!='/') strcat(ImgDir,"/");
    
    len=strlen(argv[2]);
    strcpy(OutDir,argv[2]);
    if(OutDir[len-1]!='/') strcat(OutDir,"/");
  }

  /* Initialize SDL */
  if(SDL_Init(SDL_INIT_VIDEO)<0)
  {
    fprintf(stderr,"Skinfu - Couldn't init SDL\n");
    return(-1);
  }

  printf("Loading from %s\n",ImgDir);  

  /* Load all images in the skin */
  if(LoadImageSet(SrcImages,ImgDir,ImageName))
  {
    fprintf(stderr,"Couldn't load all images\n");
    FreeImageSet(SrcImages,ImageName);
    SDL_Quit();
    return(1);
  }

  printf("SkinFU - Loaded images from %s\n",ImgDir);
  printf("SkinFU - Performing transparency processing.");

  /* Process all macros in the array */
  for(n=0; ImgMacroArray[n]!=NULL; n++)
  {
    if(ProcessMacro(SrcImages,*(ImgMacroArray[n]))<0)
      printf("\nWarning, error processing macro %d\n",n);
    else
      printf(".");
  }

  printf("\nSkinFU - Done processing.\n");
  printf("SkinFU - Saving to %s\n",OutDir);

  /* Save the new images to the output directory */
  SaveImageSet(SrcImages,OutDir,ImageName);

  printf("SkinFU - Saved.\n");

  /* Free the image set from memory */
  FreeImageSet(SrcImages,ImageName);

  /* Deinitialize SDL */
  SDL_Quit();

  /* Return success */
  printf("SkinFU - Done!\n");
  return(0);
}

