#include <stdio.h>
#include <SDL/SDL.h>

#include "skinfu.h"

#define TRANS_TEXT

int ProcessMacro(SDL_Surface *SrcImages[], ImagePos macro)
{
  int m=0;
  SDL_Surface *fg =NULL,*bg =NULL;
  SDL_Rect    fgr={0,0,0,0},bgr={0,0,0,0};

  if(macro.image==I_NONE)
    return(-1);
  else
  {
    fg=SrcImages[macro.image];
    fgr=(macro.srcrect);
  }

  if(macro.backgnd==I_NONE)
    return(-1);
  else
  {
    bg=SrcImages[macro.backgnd];
    bgr=macro.dstrect;
  }

  for(m=0; m<macro.numreps; m++)
  {
    /* Process transparency for this section */
    BlitUnder(fg,&fgr,bg,&bgr);

    /* Increment position */
    fgr.x+=macro.xoff;
    fgr.y+=macro.yoff;

    /* Special case - when yoff is 0, it can wrap back to the
       left when fgr goes beyond the edge of fg.              */
    if((fgr.x>=fg->w)&&(macro.yoff==0))
    {
      /* Wrap to the starting X position */
      fgr.x=macro.srcrect.x;
      /* Move down one row               */
      fgr.y+=fgr.h;
    }
  }

  return(0);
}


/*! \brief Saves an array of images to given directory. */
void SaveImageSet(SDL_Surface *array[], const char *path,
                  const char  *names[])
{
  int n;
  for(n=0; names[n]!=NULL; n++)
  if(array[n]!=NULL)
  {
    char buf[300];
    sprintf(buf,"%s%s",path,names[n]);
    SDL_SaveBMP(array[n],buf);
  }
}

/*! \brief Effectively blits somethint BENEATH a transparent section of fore*/
void BlitUnder(SDL_Surface *fore, SDL_Rect *fgrect, SDL_Surface *back,
               SDL_Rect *bgrect)
{
  SDL_Surface *surf=NULL;

  if((fore==NULL)||(fgrect==NULL)||(back==NULL)||(bgrect==NULL))
  {
    fprintf(stderr,"BlitUnder(%p,%p,%p,%p)\n",fore,fgrect,back,bgrect);
    return;
  }

  surf=SDL_ConvertSurface(fore,fore->format,SDL_SWSURFACE);
  if(surf==NULL)
  {
    fprintf(stderr,"Couldn't copy foregnd surface\n");
  }

  /* Clear the surface w/transparent color*/
  SDL_FillRect(surf,NULL,SDL_MapRGB(surf->format,255,0,255));

  /* Blit on the background */
  SDL_BlitSurface(back,bgrect,surf,fgrect);

  /* Blit the foreground overtop */
  SDL_BlitSurface(fore,fgrect,surf,fgrect);

  /* Blit everything back to fore */
  SDL_BlitSurface(surf,fgrect,fore,fgrect);

  /* Free temporary surface */
  SDL_FreeSurface(surf);
}

/*! \brief Frees a given image set */
void FreeImageSet(SDL_Surface *array[], const char *names[])
{
  int n=0;

  for(n=0; names[n]!=NULL; n++)
  {
    if(array[n]!=NULL)
    {
      SDL_FreeSurface(array[n]);
    }
  }
}

/*! \brief Loads an array of images, and sets transparent color to 255,0,255*/
int LoadImageSet(SDL_Surface *array[], const char *path,
                  const char *names[])
{
  int failed=0;
  int n=0;
  for(n=0; names[n]!=NULL; n++)
  {
    char buf[300];
    sprintf(buf,"%s%s",path,names[n]);

    array[n]=SDL_LoadBMP(buf);
    if(array[n]==NULL)
    {
      failed++;
      continue;
    }

    SDL_SetColorKey(array[n],SDL_SRCCOLORKEY,
                    SDL_MapRGB(array[n]->format,TRANSPARENT_COLOR));

  }

  return(failed);
} 

/*! \brief Array of images names, NULL-terminated for use with imagesets */
const char *ImageName[] = {"balance.bmp" , "cbuttons.bmp", "eq_ex.bmp"   ,
                           "eqmain.bmp"  , "main.bmp"    , "monoster.bmp",
                           "nums_ex.bmp" , "playpaus.bmp", "pledit.bmp"  ,
                           "posbar.bmp"  , "shufrep.bmp" , "text.bmp"    ,
                           "titlebar.bmp", "volume.bmp"  , NULL           };

ImagePos Rewind  =SINGLE_MAP(I_CBUTTONS,I_MAIN,  0,  0, 22, 17, 16, 88);
ImagePos RewindCl=SINGLE_MAP(I_CBUTTONS,I_MAIN,  0, 18, 22, 35, 16, 88);
ImagePos Play    =SINGLE_MAP(I_CBUTTONS,I_MAIN, 23,  0, 45, 17, 39, 88);
ImagePos PlayCl  =SINGLE_MAP(I_CBUTTONS,I_MAIN, 23, 18, 45, 35, 39, 88);
ImagePos Pause   =SINGLE_MAP(I_CBUTTONS,I_MAIN, 46,  0, 68, 17, 62, 88);
ImagePos PauseCl =SINGLE_MAP(I_CBUTTONS,I_MAIN, 46, 18, 68, 35, 62, 88);
ImagePos Stop    =SINGLE_MAP(I_CBUTTONS,I_MAIN, 69,  0, 91, 17, 85, 88);
ImagePos StopCl  =SINGLE_MAP(I_CBUTTONS,I_MAIN, 69, 18, 91, 35, 85, 88);
ImagePos FF      =SINGLE_MAP(I_CBUTTONS,I_MAIN, 92,  0,113, 17,108, 88);
ImagePos FFCl    =SINGLE_MAP(I_CBUTTONS,I_MAIN, 92, 18,113, 35,108, 88);
ImagePos Open    =SINGLE_MAP(I_CBUTTONS,I_MAIN,114,  0,135, 15,136, 89);
ImagePos OpenCl  =SINGLE_MAP(I_CBUTTONS,I_MAIN,114, 16,135, 31,136, 89);
ImagePos LpOff   =SINGLE_MAP(I_SHUFREP ,I_MAIN,  0,  0, 27, 14,210, 89);
ImagePos LpOffCl =SINGLE_MAP(I_SHUFREP ,I_MAIN,  0, 15, 27, 29,210, 89);
ImagePos LpOn    =SINGLE_MAP(I_SHUFREP ,I_MAIN,  0, 30, 27, 44,210, 89);
ImagePos LpOnCl  =SINGLE_MAP(I_SHUFREP ,I_MAIN,  0, 45, 27, 59,210, 89);
ImagePos ShfOff  =SINGLE_MAP(I_SHUFREP ,I_MAIN, 28,  0, 73, 14,164, 89);
ImagePos ShfOffCl=SINGLE_MAP(I_SHUFREP ,I_MAIN, 28, 15, 73, 29,164, 89);
ImagePos ShfOn   =SINGLE_MAP(I_SHUFREP ,I_MAIN, 28, 30, 73, 44,164, 89);
ImagePos ShfOnCl =SINGLE_MAP(I_SHUFREP ,I_MAIN, 28, 45, 73, 59,164, 89);
ImagePos EqOff   =SINGLE_MAP(I_SHUFREP ,I_MAIN,  0, 61, 22, 72,219, 58);
ImagePos EqOffCl =SINGLE_MAP(I_SHUFREP ,I_MAIN, 46, 61, 68, 72,219, 58);
ImagePos EqOn    =SINGLE_MAP(I_SHUFREP ,I_MAIN,  0, 73, 22, 84,219, 58);
ImagePos EqOnCl  =SINGLE_MAP(I_SHUFREP ,I_MAIN, 46, 73, 68, 84,219, 58);
ImagePos PlOff   =SINGLE_MAP(I_SHUFREP ,I_MAIN, 23, 61, 45, 72,242, 58);
ImagePos PlOffCl =SINGLE_MAP(I_SHUFREP ,I_MAIN, 69, 61, 91, 72,242, 58);
ImagePos PlOn    =SINGLE_MAP(I_SHUFREP ,I_MAIN, 23, 73, 45, 84,242, 58);
ImagePos PlOnCl  =SINGLE_MAP(I_SHUFREP ,I_MAIN, 69, 73, 91, 84,242, 58);
ImagePos StOn    =SINGLE_MAP(I_MONOSTER,I_MAIN,  0,  0, 28, 11,239, 41);
ImagePos StOff   =SINGLE_MAP(I_MONOSTER,I_MAIN,  0, 12, 28, 23,239, 41);
ImagePos MnOn    =SINGLE_MAP(I_MONOSTER,I_MAIN, 29,  0, 55, 11,212, 41);
ImagePos MnOff   =SINGLE_MAP(I_MONOSTER,I_MAIN, 29, 12, 55, 23,212, 41);
ImagePos PosBar  =SINGLE_MAP(I_POSBAR  ,I_MAIN,  0,  0,247,  9, 16, 72);

ImagePos PlyImg  =SINGLE_MAP(I_PLAYPAUS,I_MAIN,  1,  0,  8,  8, 27, 28);
ImagePos UDot    =SINGLE_MAP(I_PLAYPAUS,I_MAIN, 36,  0, 38,  8, 24, 28);
ImagePos LDot    =SINGLE_MAP(I_PLAYPAUS,I_MAIN, 39,  0, 41,  8, 24, 28);
ImagePos PauseImg=SINGLE_MAP(I_PLAYPAUS,I_MAIN,  9,  0, 17,  8, 26, 28);
ImagePos StopImg =SINGLE_MAP(I_PLAYPAUS,I_MAIN, 18,  0, 26,  8, 26, 28);
ImagePos BlnkSpc =SINGLE_MAP(I_PLAYPAUS,I_MAIN, 27,  0, 28,  8, 24, 28);
ImagePos Icon    =SINGLE_MAP(I_TTL     ,I_TTL ,  0,  0,  8,  8, 33,  3);
ImagePos IconCl  =SINGLE_MAP(I_TTL     ,I_TTL ,  0,  9,  8, 17, 33,  3);
ImagePos Minmz   =SINGLE_MAP(I_TTL     ,I_TTL ,  9,  0, 17,  8,281,  3);
ImagePos MinmzCl =SINGLE_MAP(I_TTL     ,I_TTL ,  9,  9, 17, 17,281,  3);
ImagePos Reduce  =SINGLE_MAP(I_TTL     ,I_TTL ,  0, 18,  8, 26,281,  3);
ImagePos ReduceCl=SINGLE_MAP(I_TTL     ,I_TTL ,  9, 18, 17, 26,281,  3);
ImagePos Rduced  =SINGLE_MAP(I_TTL     ,I_TTL ,  0, 27,  8, 35,281, 45);
ImagePos RducedCl=SINGLE_MAP(I_TTL     ,I_TTL ,  9, 27, 17, 35,281, 45);
ImagePos Quit    =SINGLE_MAP(I_TTL     ,I_TTL , 18,  0, 26,  8,291,  3);
ImagePos QuitCl  =SINGLE_MAP(I_TTL     ,I_TTL , 18,  9, 26, 17,291,  3);
ImagePos TtlFg   =SINGLE_MAP(I_TTL     ,I_MAIN, 27,  0,301, 13,  0,  0);
ImagePos TtlBg   =SINGLE_MAP(I_TTL     ,I_TTL , 27, 15,301, 28, 27,  0);
ImagePos MPosBar =SINGLE_MAP(I_TTL     ,I_TTL ,  0, 36, 16, 42,253, 46);
ImagePos BttnBar =SINGLE_MAP(I_TTL     ,I_MAIN,304,  0,311, 42, 10, 22);
ImagePos BttnBar2={I_TTL,I_MAIN,ULLR_RECT(304,44,311,86),
                   ULLR_RECT(10,22,17,64),5,8,0};

ImagePos MinEqClose=SINGLE_MAP(I_EQ_EX,I_EQ_EX,11,38,20,47,264,18);
ImagePos MinEqCloseCl=SINGLE_MAP(I_EQ_EX,I_EQ_EX,11,48,20,48+9,264,19);

ImagePos MinEqReduce=SINGLE_MAP(I_EQ_EX,I_EQ_EX,1,38,10,38+9,254,18);
ImagePos MinEqReduceCl=SINGLE_MAP(I_EQ_EX,I_EQ_EX,1,48,10,48+9,254,19);

ImagePos Number  ={I_NUMS_EX,I_MAIN,ULLR_RECT(0,0,15,24),
                   ULLR_RECT(73,52,88,76),12,16,0};
ImagePos VolBar  ={I_VOLUME,I_MAIN,ULLR_RECT(0,0,67,12),
                   ULLR_RECT(107,57,174,69),28,0,15};
ImagePos PanBar  ={I_BALANCE,I_MAIN,ULLR_RECT(9,0,46,12),
                   ULLR_RECT(177,57,214,69),28,0,15};
ImagePos Text    ={I_TEXT,I_MAIN,ULLR_RECT(0,0,4,5),
                   ULLR_RECT(111,43,115,48),31*3,5,0};

ImagePos Equ_Preset={I_EQMAIN,I_EQMAIN,{224,164,44,12},{217,18,44,12},2,0,12};
ImagePos Equ_On=    {I_EQMAIN,I_EQMAIN,{10,119,25,12}, {14,18,25,12} ,4,59,0};
ImagePos Equ_Auto  ={I_EQMAIN,I_EQMAIN,{35,119,33,12}, {39,18,39,12} ,4,59,0};
ImagePos Equ_Disp  ={I_EQMAIN,I_EQMAIN,{0,294,113,19}, {86,17,113,19},1,0 ,0};
ImagePos Equ_Close ={I_EQMAIN,I_EQMAIN,{0,116,9,9}   , {264,3,9,9}   ,2,0, 9};

ImagePos Equ_Gain1 ={I_EQMAIN,I_EQMAIN,{13,164,14,63}, {21,38,14,63},15,15,0};
ImagePos Equ_Gain2 ={I_EQMAIN,I_EQMAIN,{13,229,14,63}, {21,38,14,63},15,15,0};

ImagePos Equ_Bar   ={I_EQMAIN,I_EQMAIN,{0,134,275,15}, {0,0,275,15},2,0,15};

ImagePos Pledit_min1={I_PLEDIT,I_PLEDIT,ULLR_RECT(62,42,70,50),ULLR_RECT(157,3,165,11),1,0,0};
ImagePos Pledit_min2={I_PLEDIT,I_PLEDIT,ULLR_RECT(150,42,158,50),ULLR_RECT(128,45,136,53),1,0,0};
ImagePos Pledit_clo1={I_PLEDIT,I_PLEDIT,ULLR_RECT(52,42,60,50),ULLR_RECT(167,3,175,11),1,0,0};

/*! \brief Array of image macros, NULL-terminated */
ImagePos *ImgMacroArray[] = { /*Rewind/Play/Pause/Stop/FF/Open buttons */
                              &Rewind, &RewindCl, &Play, &PlayCl,
                              &Pause, &PauseCl, &Stop, &StopCl,
                              &FF,&FFCl,&Open, &OpenCl,

                              /* Loop, shuffle, equalizer, playlist */
                              &LpOff,&LpOffCl, &LpOn, &LpOnCl,
                              &ShfOff, &ShfOffCl, &ShfOn, &ShfOnCl,
                              &EqOff, &EqOffCl, &EqOn, &EqOnCl,
                              &PlOff, &PlOffCl, &PlOn, &PlOnCl,

                              /* Stereo on/off, mono on/off */
                              &StOn, &StOff, &MnOn, &MnOff,

                              /*Position bar, volume bar, panning bar */
                              &PosBar,&VolBar,&PanBar,
                              &MPosBar,

                              /*foreground window and bg window titlebar*/
                              &TtlFg, &TtlBg,

                              /* Vertical button bars */
                              &BttnBar,&BttnBar2,

                              /*Fullsize icon, minimize, reduce, quit buttons*/
                              &Icon,&IconCl,&Minmz,&MinmzCl,&Reduce,&ReduceCl,
                              &Rduced,&RducedCl,&Quit,&QuitCl,
#ifdef TRANS_TEXT
                              /* Letters and numbers */
                              &Text,&Number,
#endif
                              &PlyImg, &UDot, &LDot,&PauseImg,
                              &StopImg, &BlnkSpc,

                              /* Miniature equalizer buttons */
                              &MinEqClose,&MinEqCloseCl,&MinEqReduce,
                              &MinEqReduceCl,

                              /* Equalizer controls */
                              &Equ_Preset,&Equ_On,&Equ_Auto,&Equ_Disp,
                              &Equ_Gain1,&Equ_Gain2,&Equ_Close,&Equ_Bar,

                              /* Playlist controls */
                              &Pledit_min1,&Pledit_min2,&Pledit_clo1,

                              
                              NULL /* End of list */
};

