/***************************************************************************
                skinfu.h  -  type definitions for Skinfu
                             -------------------
    begin                : Sun Aug 17 10:52:47 CST 2003
    copyright            : (C) 2003 by Tyler Montbriand
    email                : tsm@accesscomm.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef __SKINFU_H__
#define __SKINFU_H__

#include <SDL/SDL.h>

#ifdef __cplusplus
extern "C" {
#endif

#define TRANSPARENT_COLOR 255,0,255

/**
 *  Macro to simplify initialization of SDL_Rects, taking
 *  upper left and lower right coordinates as parameters
 *  instead of upper-right corner and width/height.
 */
#define ULLR_RECT(x1,y1,x2,y2) {x1,y1,(x2-x1)+1,(y2-y1)+1}

/**
 *  Macro to simplify initialization of a ImagePos structure.
 *  Parameters are:
 *   -Surface in question(by ImageNum)
 *   -Background surface in question(by ImageNum)
 *   -Upper-left corner coordinates of foreground block
 *   -Lower-right corner coordinates of foreground block
 *   -Upper-left corner coordinates of background block
 */
#define SINGLE_MAP(src,bgnd,x1,y1,x2,y2,z1,z2)        \
                    {src,bgnd, ULLR_RECT(x1,y1,x2,y2),\
                    {z1,z2,(x2-x1)+1,(y2-y1)+1},1,0,0}

/*! \brief Array of image names corresponding to ImageNum */
extern const char *ImageName[];

/*! \brief Frees a set of images */
void FreeImageSet(SDL_Surface *array[], const char *names[]);
/**
 * Attempts to load a list of images into 'array', from
 * a NULL-terminated array of strings 'names', in path
 * 'path'.
 */ 
int LoadImageSet(SDL_Surface *array[], const char *path,
                 const char *names[]);
/**
 * Attempts to save a set of images to file, from 'array',
 * with names specified in the NULL-terminated string array
 * 'names', in path 'path'.
 */                 
void SaveImageSet(SDL_Surface *array[], const char *path,
                  const char *names[]);

/**
 * Attempts to blit "underneath" the specified portion of
 * 'fore', replacing the transparent(RGB255,0,255) portions
 * with the specified background.
 */
void BlitUnder(SDL_Surface *fore, SDL_Rect *fr, SDL_Surface *back,
               SDL_Rect *bgr);

/**
 * An enumerated type corresponding to the list of names in
 * ImageName.
 */               
typedef enum ImageNum
{
  I_NONE    = -1,
  /*! \brief Index of balance.bmp   */
  I_BALANCE =  0,
  /*! \brief Index of cbuttons.bmp  */
  I_CBUTTONS,
  /*! \brief Index of eq_ex.bmp     */
  I_EQ_EX,
  /*! \brief Index of eqmain.bmp    */
  I_EQMAIN,
  /*! \brief Index of main.bmp      */
  I_MAIN,
  /*! \brief Index of monoster.bmp  */
  I_MONOSTER,
  /*! \brief Index of nums_ex.bmp   */
  I_NUMS_EX,
  /*! \brief Index of playpaus.bmp  */
  I_PLAYPAUS,
  /*! \brief Index of pledit.bmp    */
  I_PLEDIT,
  /*! \brief Index of posbar.bmp    */
  I_POSBAR,
  /*! \brief Index of shufrep.bmp   */
  I_SHUFREP,
  /*! \brief Index of text.bmp      */
  I_TEXT,
  /*! \brief Index of titlebar.bmp  */
  I_TTL,
  /*! \brief Index of volume.bmp    */
  I_VOLUME,
  I_END
} ImageNum;


/*! \brief Structure defining a transparency-processing macro */
typedef	struct ImagePos {
  /*! \brief Image number of foreground */
  ImageNum image;
  /*! \brief Image number of background */
  ImageNum backgnd;
  /*! \brief Rectangular area of foreground to process */
  SDL_Rect srcrect;
  /*! \brief Rectangular area of BG image to use as background */
  SDL_Rect dstrect;
  /*! \brief The number of repititions */
  int numreps;
  /*! \brief Amount to increment srcrect's x position per rep */
  int xoff;
  /*! \brief Amount to increment srcrect's y position per rep */
  int yoff;
} ImagePos;

/*! \brief Performs one image macro on an image set */
int ProcessMacro(SDL_Surface *SrcImages[], ImagePos macro);

/*! \brief Array of every image processing macro */
extern ImagePos *ImgMacroArray[];

#ifdef __cplusplus
  }
#endif

#endif/*__SKINFU_H__*/
